@@ -63,23 +63,14 @@ let cachedUserFilters = '';
     let hintUpdateToken = 0;
 
     const getHints = async function() {
-        const response = await vAPI.messaging.send('dashboard', {
+        const hints = await vAPI.messaging.send('dashboard', {
             what: 'getAutoCompleteDetails',
             hintUpdateToken
         });
-        if ( response instanceof Object === false ) { return; }
-        if ( response.hintUpdateToken !== undefined ) {
-            const mode = cmEditor.getMode();
-            if ( typeof response.filterOnHeaders === 'boolean' ) {
-                cmEditor.setOption('filterOnHeaders', response.filterOnHeaders);
-            }
-            if ( mode.setHints instanceof Function ) {
-                mode.setHints(response);
-            }
-            if ( hintUpdateToken === 0 ) {
-                mode.parser.expertMode = response.expertMode !== false;
-            }
-            hintUpdateToken = response.hintUpdateToken;
+        if ( hints instanceof Object === false ) { return; }
+        if ( hints.hintUpdateToken !== undefined ) {
+            cmEditor.setOption('uboHints', hints);
+            hintUpdateToken = hints.hintUpdateToken;
         }
         timer.on(2503);
     };
@@ -91,6 +82,12 @@ let cachedUserFilters = '';
     getHints();
 }
 
+vAPI.messaging.send('dashboard', {
+    what: 'getTrustedScriptletTokens',
+}).then(tokens => {
+    cmEditor.setOption('trustedScriptletTokens', tokens);
+});
+
 /******************************************************************************/
 
 function getEditorText() {
@@ -169,6 +166,8 @@ async function renderUserFilters(merge = false) {
     });
     if ( details instanceof Object === false || details.error ) { return; }
 
+    cmEditor.setOption('trustedSource', details.trustedSource === true);
+
     const newContent = details.content.trim();
 
     if ( merge && self.hasUnsavedData() ) {
