@@ -149,7 +149,7 @@ export default new Map([
     [ 'noop-0.5s.mp3', {
     } ],
     [ 'noop-1s.mp4', {
-        alias: 'noopmp4-1s',
+        alias: [ 'noopmp4-1s', 'abp-resource:blank-mp4' ],
         data: 'blob',
     } ],
     [ 'noop.css', {
@@ -162,6 +162,10 @@ export default new Map([
         alias: [ 'noopjs', 'abp-resource:blank-js' ],
         data: 'text',
     } ],
+    [ 'noop.json', {
+        alias: [ 'noopjson' ],
+        data: 'text',
+    } ],
     [ 'noop.txt', {
         alias: 'nooptext',
         data: 'text',
