@@ -130,8 +130,7 @@ const fromNetFilter = async function(rawFilter) {
 
     const writer = new CompiledListWriter();
     const parser = new sfp.AstFilterParser({
-        expertMode: true,
-        filterOnHeaders: true,
+        trustedSource: true,
         maxTokenLength: staticNetFilteringEngine.MAX_TOKEN_LENGTH,
         nativeCssHas: vAPI.webextFlavor.env.includes('native_css_has'),
     });
@@ -169,7 +168,7 @@ const fromExtendedFilter = async function(details) {
     const hostname = hostnameFromURI(details.url);
 
     const parser = new sfp.AstFilterParser({
-        expertMode: true,
+        trustedSource: true,
         nativeCssHas: vAPI.webextFlavor.env.includes('native_css_has'),
     });
     parser.parse(details.rawFilter);
