@@ -147,6 +147,7 @@ const matchBucket = function(url, hostname, bucket, start) {
         }
         bucket.push(directive);
         this.saveWhitelist();
+        µb.filteringBehaviorChanged({ hostname: targetHostname });
         return true;
     }
 
@@ -187,6 +188,7 @@ const matchBucket = function(url, hostname, bucket, start) {
         }
     }
     this.saveWhitelist();
+    µb.filteringBehaviorChanged({ direction: 1 });
     return true;
 };
 
@@ -421,7 +423,7 @@ const matchBucket = function(url, hostname, bucket, start) {
         redirectEngine.invalidateResourcesSelfie(io);
         this.loadRedirectResources();
     }
-    this.fireDOMEvent('hiddenSettingsChanged');
+    this.fireEvent('hiddenSettingsChanged');
 };
 
 /******************************************************************************/
@@ -465,7 +467,8 @@ const matchBucket = function(url, hostname, bucket, start) {
 // (but not really) redundant rules led to this issue.
 
 µb.toggleFirewallRule = function(details) {
-    let { srcHostname, desHostname, requestType, action } = details;
+    const { desHostname, requestType, action } = details;
+    let { srcHostname } = details;
 
     if ( action !== 0 ) {
         sessionFirewall.setCell(
@@ -495,8 +498,7 @@ const matchBucket = function(url, hostname, bucket, start) {
             permanentFirewall.unsetCell(
                 srcHostname,
                 desHostname,
-                requestType,
-                action
+                requestType
             );
         }
         this.savePermanentFirewallRules();
@@ -521,6 +523,12 @@ const matchBucket = function(url, hostname, bucket, start) {
     // https://github.com/chrisaljoudi/uBlock/issues/420
     cosmeticFilteringEngine.removeFromSelectorCache(srcHostname, 'net');
 
+    // Flush caches
+    µb.filteringBehaviorChanged({
+        direction: action === 1 ? 1 : 0,
+        hostname: srcHostname,
+    });
+
     if ( details.tabId === undefined ) { return; }
 
     if ( requestType.startsWith('3p') ) {
@@ -566,31 +574,53 @@ const matchBucket = function(url, hostname, bucket, start) {
 /******************************************************************************/
 
 µb.toggleHostnameSwitch = function(details) {
+    const newState = typeof details.state === 'boolean'
+        ? details.state
+        : sessionSwitches.evaluateZ(details.name, details.hostname) === false;
     let changed = sessionSwitches.toggleZ(
         details.name,
         details.hostname,
         !!details.deep,
-        details.state
+        newState
     );
     if ( changed === false ) { return; }
 
-    // Take action if needed
+    // Take per-switch action if needed
     switch ( details.name ) {
-    case 'no-scripting':
-        this.updateToolbarIcon(details.tabId, 0b100);
-        break;
-    case 'no-cosmetic-filtering':
-        this.scriptlets.injectDeep(
-            details.tabId,
-            details.state ? 'cosmetic-off' : 'cosmetic-on'
-        );
-        break;
-    case 'no-large-media':
-        const pageStore = this.pageStoreFromTabId(details.tabId);
-        if ( pageStore !== null ) {
-            pageStore.temporarilyAllowLargeMediaElements(!details.state);
+        case 'no-scripting':
+            this.updateToolbarIcon(details.tabId, 0b100);
+            break;
+        case 'no-cosmetic-filtering': {
+            const scriptlet = newState ? 'cosmetic-off' : 'cosmetic-on';
+            vAPI.tabs.executeScript(details.tabId, {
+                file: `/js/scriptlets/${scriptlet}.js`,
+                allFrames: true,
+            });
+            break;
+        }
+        case 'no-large-media':
+            const pageStore = this.pageStoreFromTabId(details.tabId);
+            if ( pageStore !== null ) {
+                pageStore.temporarilyAllowLargeMediaElements(!newState);
+            }
+            break;
+        default:
+            break;
+    }
+
+    // Flush caches if needed
+    if ( newState ) {
+        switch ( details.name ) {
+            case 'no-scripting':
+            case 'no-remote-fonts':
+                µb.filteringBehaviorChanged({
+                    direction: details.state ? 1 : 0,
+                    hostname: details.hostname,
+                });
+                break;
+            default:
+                break;
         }
-        break;
     }
 
     if ( details.persist !== true ) { return; }
@@ -599,7 +629,7 @@ const matchBucket = function(url, hostname, bucket, start) {
         details.name,
         details.hostname,
         !!details.deep,
-        details.state
+        newState
     );
     if ( changed ) {
         this.saveHostnameSwitches();
@@ -627,7 +657,7 @@ const matchBucket = function(url, hostname, bucket, start) {
     return bits;
 };
 
-µb.parseBlockingProfiles = (( ) => {
+{
     const parse = function() {
         const s = µb.hiddenSettings.blockingProfiles;
         const profiles = [];
@@ -647,78 +677,20 @@ const matchBucket = function(url, hostname, bucket, start) {
 
     parse();
 
-    self.addEventListener('hiddenSettingsChanged', ( ) => { parse(); });
-
-    return parse;
-})();
+    µb.onEvent('hiddenSettingsChanged', ( ) => { parse(); });
+}
 
 /******************************************************************************/
 
-µb.scriptlets = (function() {
-    const pendingEntries = new Map();
-
-    const Entry = class {
-        constructor(tabId, scriptlet, callback) {
-            this.tabId = tabId;
-            this.scriptlet = scriptlet;
-            this.callback = callback;
-            this.timer = vAPI.setTimeout(this.service.bind(this), 1000);
+µb.pageURLFromMaybeDocumentBlockedURL = function(pageURL) {
+    if ( pageURL.startsWith(vAPI.getURL('/document-blocked.html?')) ) {
+        try {
+            const url = new URL(pageURL);
+            return JSON.parse(url.searchParams.get('details')).url;
+        } catch(ex) {
         }
-        service(response) {
-            if ( this.timer !== null ) {
-                clearTimeout(this.timer);
-                this.timer = null;
-            }
-            pendingEntries.delete(makeKey(this.tabId, this.scriptlet));
-            this.callback(response);
-        }
-    };
-
-    const makeKey = function(tabId, scriptlet) {
-        return tabId + ' ' + scriptlet;
-    };
-
-    const report = function(tabId, scriptlet, response) {
-        const key = makeKey(tabId, scriptlet);
-        const entry = pendingEntries.get(key);
-        if ( entry === undefined ) { return; }
-        entry.service(response);
-    };
-
-    const inject = function(tabId, scriptlet, callback) {
-        if ( typeof callback === 'function' ) {
-            if ( vAPI.isBehindTheSceneTabId(tabId) ) {
-                callback();
-                return;
-            }
-            const key = makeKey(tabId, scriptlet);
-            const entry = pendingEntries.get(key);
-            if ( entry !== undefined ) {
-                if ( callback !== entry.callback ) {
-                    callback();
-                }
-                return;
-            }
-            pendingEntries.set(key, new Entry(tabId, scriptlet, callback));
-        }
-        vAPI.tabs.executeScript(tabId, {
-            file: `/js/scriptlets/${scriptlet}.js`
-        });
-    };
-
-    // TODO: think about a callback mechanism.
-    const injectDeep = function(tabId, scriptlet) {
-        vAPI.tabs.executeScript(tabId, {
-            file: `/js/scriptlets/${scriptlet}.js`,
-            allFrames: true
-        });
-    };
-
-    return {
-        inject: inject,
-        injectDeep: injectDeep,
-        report: report
-    };
-})();
+    }
+    return pageURL;
+};
 
 /******************************************************************************/
