@@ -19,11 +19,12 @@
     Home: https://github.com/gorhill/uBlock
 */
 
-/* global CodeMirror, uDom, uBlockDashboard */
+/* global CodeMirror, uBlockDashboard */
 
 'use strict';
 
 import { i18n$ } from './i18n.js';
+import { dom, qs$ } from './dom.js';
 
 /******************************************************************************/
 
@@ -49,9 +50,7 @@ CodeMirror.defineMode("ubo-whitelist-directives", function() {
                 return null;
             }
             if ( reComment.test(line) ) {
-                return whitelistDefaultSet.has(directiveFromLine(line))
-                    ? 'keyword comment'
-                    : 'comment';
+                return 'comment';
             }
             if ( line.indexOf('/') === -1 ) {
                 if ( reBadHostname.test(line) ) { return 'error'; }
@@ -90,15 +89,12 @@ const noopFunc = function(){};
 
 let cachedWhitelist = '';
 
-const cmEditor = new CodeMirror(
-    document.getElementById('whitelist'),
-    {
-        autofocus: true,
-        lineNumbers: true,
-        lineWrapping: true,
-        styleActiveLine: true,
-    }
-);
+const cmEditor = new CodeMirror(qs$('#whitelist'), {
+    autofocus: true,
+    lineNumbers: true,
+    lineWrapping: true,
+    styleActiveLine: true,
+});
 
 uBlockDashboard.patchCodeMirrorEditor(cmEditor);
 
@@ -116,12 +112,12 @@ const setEditorText = function(text) {
 /******************************************************************************/
 
 const whitelistChanged = function() {
-    const whitelistElem = uDom.nodeFromId('whitelist');
-    const bad = whitelistElem.querySelector('.cm-error') !== null;
+    const whitelistElem = qs$('#whitelist');
+    const bad = qs$(whitelistElem, '.cm-error') !== null;
     const changedWhitelist = getEditorText().trim();
     const changed = changedWhitelist !== cachedWhitelist;
-    uDom.nodeFromId('whitelistApply').disabled = !changed || bad;
-    uDom.nodeFromId('whitelistRevert').disabled = !changed;
+    qs$('#whitelistApply').disabled = !changed || bad;
+    qs$('#whitelistRevert').disabled = !changed;
     CodeMirror.commands.save = changed && !bad ? applyChanges : noopFunc;
 };
 
@@ -188,7 +184,7 @@ const handleImportFilePicker = function() {
 /******************************************************************************/
 
 const startImportFilePicker = function() {
-    const input = document.getElementById('importFilePicker');
+    const input = qs$('#importFilePicker');
     // Reset to empty string, this will ensure an change event is properly
     // triggered if the user pick a file, even if it is the same as the last
     // one picked.
@@ -251,11 +247,11 @@ self.hasUnsavedData = function() {
 
 /******************************************************************************/
 
-uDom('#importWhitelistFromFile').on('click', startImportFilePicker);
-uDom('#importFilePicker').on('change', handleImportFilePicker);
-uDom('#exportWhitelistToFile').on('click', exportWhitelistToFile);
-uDom('#whitelistApply').on('click', ( ) => { applyChanges(); });
-uDom('#whitelistRevert').on('click', revertChanges);
+dom.on('#importWhitelistFromFile', 'click', startImportFilePicker);
+dom.on('#importFilePicker', 'change', handleImportFilePicker);
+dom.on('#exportWhitelistToFile', 'click', exportWhitelistToFile);
+dom.on('#whitelistApply', 'click', ( ) => { applyChanges(); });
+dom.on('#whitelistRevert', 'click', revertChanges);
 
 renderWhitelist();
 
